@@ -1,6 +1,10 @@
 # coding=utf-8
 from __future__ import (absolute_import, division, unicode_literals)
 
+from django.conf import settings
+if not getattr(settings, "HYPERGEN_ENABLE_OLD", False):
+    raise Exception("This is the old version of hypergen. Please use template.py, liveview.py, etc.")
+
 d = dict
 
 import string, sys, time, threading, datetime, json, logging
@@ -9,10 +13,11 @@ from types import GeneratorType
 from functools import wraps, update_wrapper
 from copy import deepcopy
 
-from contextlib import ContextDecorator, contextmanager
+from contextlib2 import ContextDecorator, contextmanager
 from pyrsistent import pmap, m
 
 from django.http.response import HttpResponse, HttpResponseRedirect
+
 try:
     from django.utils.encoding import force_text
 except ImportError:
@@ -39,7 +44,7 @@ __all__ = [
     "script", "section", "select", "small", "source", "span", "strike", "strong", "style", "sub", "summary", "sup",
     "svg", "table", "tbody", "td", "template", "textarea", "tfoot", "th", "thead", "time", "title", "tr", "track",
     "tt", "u", "ul", "var", "video", "wbr", "component", "hypergen", "command", "raw", "callback", "call_js", "THIS",
-    "OMIT", "context", "is_ajax", "write"]
+    "OMIT", "context", "is_ajax"]
 
 ### Python 2+3 compatibility ###
 
@@ -119,12 +124,6 @@ def wrap2(f):
 def insert(source_str, insert_str, pos):
     return ''.join((source_str[:pos], insert_str, source_str[pos:]))
 
-def is_ajax(request=None):
-    if request is None:
-        request = c.request
-
-    return request.META.get('HTTP_X_REQUESTED_WITH', None) == 'XMLHttpRequest'
-
 ### Rendering ###
 
 def default_wrap_elements(init, self, *children, **attrs):
@@ -177,8 +176,7 @@ def hypergen(func, *args, **kwargs):
             print(("Execution time:", (time.time() - a) * 1000, "ms"))
             return html
         else:
-            if c.hypergen.target_id != OMIT:
-                command("hypergen.morph", c.hypergen.target_id, html)
+            command("hypergen.morph", c.hypergen.target_id, html)
             print(("Execution time:", (time.time() - a) * 1000, "ms"))
             return c.hypergen.commands
 
@@ -265,9 +263,6 @@ def join_html(html):
 def raw(*children):
     c.hypergen.into.extend(children)
 
-def write(*children):
-    c.hypergen.into.extend(t(x) for x in children)
-
 def t(s, quote=True, translatable=False):
     return translate(escape(make_string(s), quote=quote), translatable=translatable)
 
@@ -376,7 +371,7 @@ class THIS(object):
 NON_SCALARS = set((list, dict, tuple))
 DELETED = ""
 
-COERCE = {str: "hypergen.coerce.str", int: "hypergen.coerce.int", float: "hypergen.coerce.float"}
+COERCE = {str: None, str: None, int: "hypergen.coerce.int", float: "hypergen.coerce.float"}
 
 class base_element(ContextDecorator):
     void = False
@@ -446,10 +441,7 @@ class base_element(ContextDecorator):
                 name, a, kw = v.hypergen_callback_signature
                 return "{}({})".format(name, signature(a, kw))
             elif callable(v):
-                if hasattr(v, "__name__"):
-                    return ".".join((v.__module__, v.__name__)).replace("builtins.", "")
-                else:
-                    return repr(v)
+                return ".".join((v.__module__, v.__name__)).replace("builtins.", "")
             elif type(v) is str:
                 return '"{}"'.format(v)
             else:
@@ -830,14 +822,6 @@ class wbr(base_element_void): pass
 # yapf: enable
 
 # Serialization
-ENCODINGS = {
-    datetime.date: lambda o: {"_": ["date", str(o)]},
-    datetime.datetime: lambda o: {"_": ["datetime", str(o)]},
-    tuple: lambda o: {"_": ["tuple", list(o)]},
-    set: lambda o: {"_": ["set", list(o)]},
-    frozenset: lambda o: {"_": ["frozenset", list(o)]},
-    range: lambda o: {"_": ["range", [o.start, o.stop, o.step]]},}
-
 def encoder(o):
     assert not hasattr(o, "reverse"), "Should not happen"
     if issubclass(type(o), base_element):
@@ -846,21 +830,15 @@ def encoder(o):
     elif hasattr(o, "__weakref__"):
         # Lazy strings and urls.
         return make_string(o)
-    fn = ENCODINGS.get(type(o), None)
-    if fn:
-        return fn(o)
+    if type(o) is datetime.date:
+        return {"_": ["date", str(o)]}
     else:
         raise TypeError(repr(o) + " is not JSON serializable")
 
-DECODINGS = {
-    "float": float,
-    "date": parse_date,
-    "datetime": parse_datetime,
-    "time": parse_time,
-    "tuple": tuple,
-    "set": set,
-    "frozenset": frozenset,
-    "range": lambda v: range(*v),}
+def dumps(data, default=encoder, indent=None):
+    result = json.dumps(data, default=encoder, separators=(',', ':'), indent=indent)
+
+    return result
 
 def decoder(o):
     _ = o.get("_", None)
@@ -868,18 +846,18 @@ def decoder(o):
         return o
 
     datatype, value = _
-    fn = DECODINGS.get(datatype, None)
-    if fn:
-        return fn(value)
+    if datatype == "float":
+        return float(value)
+    elif datatype == "date":
+        return parse_date(value)
+    elif datatype == "datetime":
+        return parse_datetime(value)
+    elif datatype == "time":
+        return parse_time(value)
     else:
 
         raise Exception("Unknown datatype, {}".format(datatype))
 
-def dumps(data, default=encoder, indent=None):
-    result = json.dumps(data, default=encoder, separators=(',', ':'), indent=indent)
-
-    return result
-
 def loads(data):
     return json.loads(data, object_hook=decoder)
 
@@ -937,29 +915,17 @@ class Context(threading.local):
     @contextmanager
     def __call__(self, transformer=None, at=None, **items):
         try:
-            # Store previous value.
             ctx = self.ctx
             if at is None:
                 if transformer is not None:
                     self.ctx = transformer(self.ctx)
                 self.ctx = self.ctx.update(m(**items))
             else:
-                if at not in self.ctx:
-                    self.ctx = self.ctx.set(at, pmap(items))
-                else:
-                    new_value_at = self.ctx[at].update(pmap(items))
-                    if not new_value_at:
-                        raise Exception("Not immutable context variable attempted updated. If you want to nest "
-                            "with context() statements you must use a pmap() or another immutable hashmap type.")
-
-                    self.ctx = self.ctx.set(at, new_value_at)
-
                 if transformer is not None:
                     self.ctx = self.ctx.set(at, transformer(self.ctx[at]))
-
+                self.ctx = self.ctx.set(at, self.ctx[at].update(m(**items)))
             yield
         finally:
-            # Reset to previous value.
             self.ctx = ctx
 
 context = Context()
@@ -977,4 +943,11 @@ def context_middleware(get_response):
 
 class ContextMiddleware(MiddlewareMixin):
     def process_request(self, request):
+        # TODO. Change to MIDDLEWARE and not MIDDLEWARE_CLASSES
         context.replace(**_init_context(request))
+
+def is_ajax(request=None):
+    if request is None:
+        request = c.request
+
+    return request.META.get('HTTP_X_REQUESTED_WITH', None) == 'XMLHttpRequest'
