@@ -1,6 +1,3 @@
-# coding=utf-8
-from __future__ import (absolute_import, division, unicode_literals)
-
 import pytest
 from hypergen.incubation import SessionVar, pickle_dumps
 
